using Adaptation.Shared;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Text;
using System.Text.Json;
using System.Text.Json.Serialization;

namespace Adaptation.FileHandlers.txt;

#nullable enable

internal class Run
{

    public Run(Header header, Summary summary, ReadOnlyCollection<Wafer> wafers)
    {
        Header = header;
        Summary = summary;
        Wafers = wafers;
    }

    public Header Header { get; }
    public Summary Summary { get; }
    public ReadOnlyCollection<Wafer> Wafers { get; }

    private static void WriteJson(Logistics logistics, List<FileInfo> fileInfoCollection, Run result)
    {
        FileInfo fileInfo = new($"{logistics.ReportFullPath}.json");
        string json = JsonSerializer.Serialize(result, RunSourceGenerationContext.Default.Run);
        File.WriteAllText(fileInfo.FullName, json);
        File.SetLastWriteTime(fileInfo.FullName, logistics.DateTimeFromSequence);
        fileInfoCollection.Add(fileInfo);
    }

    private static ReadOnlyCollection<string> GetLines(JsonElement[]? jsonElements)
    {
        List<string> results = new();
        int columns = 0;
        StringBuilder stringBuilder = new();
        for (int i = 0; i < jsonElements?.Length;)
        {
            foreach (JsonProperty jsonProperty in jsonElements[0].EnumerateObject())
            {
                columns += 1;
                _ = stringBuilder.Append('"').Append(jsonProperty.Name).Append('"').Append('\t');
            }
            break;
        }
        if (jsonElements?.Length != 0)
            _ = stringBuilder.Remove(stringBuilder.Length - 1, 1);
        results.Add(stringBuilder.ToString());
        for (int i = 0; i < jsonElements?.Length; i++)
        {
            _ = stringBuilder.Clear();
            foreach (JsonProperty jsonProperty in jsonElements[i].EnumerateObject())
            {
                if (jsonProperty.Value.ValueKind == JsonValueKind.Object)
                    _ = stringBuilder.Append('\t');
                else if (jsonProperty.Value.ValueKind != JsonValueKind.String)
                    _ = stringBuilder.Append(jsonProperty.Value).Append('\t');
                else
                    _ = stringBuilder.Append('"').Append(jsonProperty.Value).Append('"').Append('\t');
            }
            _ = stringBuilder.Remove(stringBuilder.Length - 1, 1);
            results.Add(stringBuilder.ToString());
        }
        return results.AsReadOnly();
    }

    private static ReadOnlyCollection<string> GetLines(JsonElement jsonElement) =>
        GetLines(new JsonElement[] { jsonElement });

    private static void WriteTabSeparatedValues(Logistics logistics, List<FileInfo> fileInfoCollection, Run run)
    {
        List<Row> results = new();
        Row row;
        FileInfo fileInfo = new($"{logistics.ReportFullPath}.tsv");
        for (int i = 0; i < run.Wafers.Count; i++)
        {
            row = new(run, i);
            results.Add(row);
        }
        string json = JsonSerializer.Serialize(results);
        JsonElement[]? jsonElements = JsonSerializer.Deserialize<JsonElement[]>(json);
        ReadOnlyCollection<string> lines = GetLines(jsonElements);
        File.WriteAllText(fileInfo.FullName, string.Join(Environment.NewLine, lines));
        File.SetLastWriteTime(fileInfo.FullName, logistics.DateTimeFromSequence);
        fileInfoCollection.Add(fileInfo);
    }

    private static void WriteException(Logistics logistics, Exception ex)
    {
        FileInfo fileInfo = new($"{logistics.ReportFullPath}.{nameof(Exception)}.txt");
        File.WriteAllText(fileInfo.FullName, $"{ex.Message}{Environment.NewLine}{ex.StackTrace}");
    }

    internal static Run? Get(Logistics logistics, List<FileInfo> fileInfoCollection)
    {
        Run? result;
        string text = File.ReadAllText(logistics.ReportFullPath);
        if (string.IsNullOrEmpty(text))
            result = null;
        else
        {
            Constant constant = new();
            int[] i = new int[] { 0 };
            List<WaferSummaryInfo> dcnTotals = new();
            List<WaferSummaryInfo> dwnTotals = new();
            List<WaferSummaryInfo> dnnTotals = new();
            Header header = Header.Get(constant, i, text);
            Summary summary = Summary.Get(constant, i, text, dcnTotals, dwnTotals, dnnTotals);
            ReadOnlyCollection<Wafer> wafers = Wafer.Get(constant, dcnTotals, dwnTotals, dnnTotals);
            if (wafers.Count == 0)
                result = null;
            else
            {
                result = new(header, summary, wafers);
                WriteJson(logistics, fileInfoCollection, result);
                try
                {
                    WriteTabSeparatedValues(logistics, fileInfoCollection, result);
                }
                catch (Exception ex)
                {
                    WriteException(logistics, ex);
                }
            }
        }
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(Run))]
internal partial class RunSourceGenerationContext : JsonSerializerContext
{
}